/*  
    ACSL specification for strcat function from Linux lib/string.c
    Copyright (c) 2018 Denis Efremov <efremov@ispras.ru>
    Copyright (c) 2018 ISP RAS (http://www.ispras.ru)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include "strlen.h"

/**
 * strcat - Append one %NUL-terminated string to another
 * @dest: The string to be appended to
 * @src: The string to append to it
 */

/*@ requires valid_str(src);
    requires valid_str(dest);
    requires strlen(dest)+strlen(src) <= SIZE_MAX;
    requires \valid(dest+(0..strlen(dest)+strlen(src)));
    assigns dest[strlen(dest)..strlen(dest)+strlen(src)];
    ensures \result == dest;
    ensures valid_str(\result);
    ensures \forall integer i; 0 <= i < strlen{Old}(dest) ==>
            \old(dest[i]) == \result[i];
    ensures \forall integer i;
            strlen{Old}(dest) <= i < strlen{Old}(dest) + strlen(src) ==>
            src[i - strlen{Old}(dest)] == \result[i];
    ensures strlen(\result) == strlen{Old}(dest) + strlen(src);
 */
char *strcat(char *dest, const char *src)
{
	char *tmp = dest;
	//@ ghost size_t dest_len = strlen(tmp);

	/*@ loop invariant tmp <= dest <= tmp + dest_len;
	    loop invariant valid_str(dest);
	    loop invariant \forall integer i; 0 <= i < dest - tmp ==> tmp[i] != '\0';
	    loop variant dest_len - (dest - tmp);
	 */
	while (*dest)
		dest++;
	//@ assert *dest == '\0';
	//@ assert dest == tmp + dest_len;
	//@ assert strlen(tmp) == dest_len;
	//@ ghost char *osrc = src;
	//@ ghost char *mdest = dest;

	/*@ loop invariant osrc <= src <= osrc + strlen(osrc);
	    loop invariant mdest <= dest <= mdest + strlen(osrc);
	    loop invariant src - osrc == dest - mdest;
	    loop invariant valid_str(src);
	    loop invariant \forall integer i; 0 <= i < src - osrc ==>
	                   mdest[i] == osrc[i];
	    loop invariant \forall integer i; 0 <= i < dest - tmp ==> tmp[i] != '\0';
	    loop assigns mdest[0..strlen(osrc)];
	    loop variant strlen(osrc) - (src - osrc);
	 */
	while ((*dest++ = *src++) != '\0')
		;
	//@ assert \forall integer i; 0 <= i < dest_len ==> \at(dest[i],Pre) == tmp[i];
	//@ assert dest[-1] == '\0' && src[-1] == '\0';
	//@ assert dest - 1 == tmp + dest_len + strlen(osrc);
	//@ assert strlen(osrc) == src - osrc - 1;
	//@ assert \exists size_t n; tmp[n] == '\0' && \valid(tmp+(0..n)) && n == (size_t) (dest_len + strlen(osrc));
	/*@ assert valid_str(tmp) &&
	    (tmp[(size_t)(dest_len + strlen(osrc))] == '\0') &&
	    (\forall integer i; 0 <= i < (size_t)(dest_len + strlen(osrc)) ==> tmp[i] != '\0');
	 */
	//@ assert strlen(tmp) == (size_t) (dest_len + strlen(osrc));
	return tmp;
}
