#ifndef __STRLEN_H__
#define __STRLEN_H__

/*  
    ACSL specification for strlen function from Linux lib/string.c
    Copyright (c) 2018 Denis Efremov <efremov@ispras.ru>
    Copyright (c) 2018 ISP RAS (http://www.ispras.ru)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */



#define SIZE_MAX 18446744073709551615UL

typedef unsigned long __kernel_ulong_t;
typedef __kernel_ulong_t __kernel_size_t;
typedef __kernel_size_t size_t;

/*@ axiomatic Strlen {
    predicate valid_str(char *s) =
       \exists size_t n;
          s[n] == '\0' && \valid(s+(0..n));
    lemma valid_str_shift1:
       \forall char *s;
          *s != '\0' &&
          valid_str(s) ==>
             valid_str(s+1);
    lemma valid_str_strend:
       \forall char *s;
          \valid(s) && *s == '\0' ==>
             valid_str(s);
    logic size_t strlen(char *s) =
       s[0] == '\0' ? (size_t) 0 : (size_t) ((size_t)1 + strlen(s + 1));
    lemma strlen_before_null:
       \forall char* s, integer i;
          valid_str(s) &&
          0 <= i < strlen(s) ==> s[i] != '\0';
    lemma strlen_at_null:
       \forall char* s;
          valid_str(s) ==> s[strlen(s)] == '\0';
    lemma strlen_shift1:
       \forall char *s;
          valid_str(s) && *s != '\0' ==>
          strlen(s) == 1 + strlen(s+1);
    lemma strlen_main:
       \forall char *s, size_t n;
       valid_str(s) &&
       s[n] == '\0' &&
       (\forall integer i; 0 <= i < n ==> s[i] != '\0') ==>
           strlen(s) == n;
    }
 */


/**
 * strlen - Find the length of a string
 * @s: The string to be sized
 */

/*@ requires valid_str(s);
    assigns \nothing;
    ensures \result == strlen(s);
    ensures s[\result] == '\0';
    ensures \forall integer i; 0 <= i < \result ==> s[i] != '\0';
 */
size_t strlen(const char *s);

#endif // __STRLEN_H__
